//
// main.c
//
// Emic 2-to-Elmo
//
// This file contains code for the Microchip PIC12F675 for CCS PCM
//

/****************************************************************************************
   This program serves as a simple interface between the Emic 2 Text-to-Speech Module
   (Parallax #30016, http://www.parallax.com) and the Elmo Play-a-Song toy. The three
   buttons on the Elmo toy (Play, Shuffle, Stop) will play the three demonstrations 
   built into Emic 2. The Shuffle and Stop buttons can be used during playback to 
   pause or stop, respectively. 
   
   Written by Joe Grand, Grand Idea Studio, Inc. [www.grandideastudio.com]
   
   Distributed under a Creative Commons Attribution 3.0 US License
*****************************************************************************************/

#include "elmo.h"

void main(void)
{
	state_type state = IDLE; // Default state
	hardware_init();
	emic2_init();

    while(1)
	{	
		switch (state)
		{
			case IDLE:	// Poll for button press
				if (!input(SW_PLAY))
				{
					delay_ms(DB_TIME); 			// Poor man's debounce
					while (!input(SW_PLAY)); 	// Wait for button to be released
					printf("D0\n");				// Play demonstration 0
					state = PLAYING;
				}
				else if (!input(SW_SHUFFLE))
				{
					delay_ms(DB_TIME); 			// Poor man's debounce
					while (!input(SW_SHUFFLE)); // Wait for button to be released
					printf("D1\n");				// Play demonstration 1
					state = PLAYING;
				}				
				else if (!input(SW_STOP))
				{
					delay_ms(DB_TIME); 			// Poor man's debounce
					while (!input(SW_STOP)); 	// Wait for button to be released
					printf("D2\n");				// Play demonstration 2
					state = PLAYING;
				}
				break;	
			case PLAYING:  // While the Emic 2 is speaking... 
				while (!emic2_check())
				{
					if (!input(SW_SHUFFLE))
					{
						delay_ms(DB_TIME); 			// Poor man's debounce
						while (!input(SW_SHUFFLE)); // Wait for button to be released				
						printf("Z\n");				// Send Pause command						
					}
					else if (!input(SW_STOP))
					{
						delay_ms(DB_TIME); 			// Poor man's debounce
						while (!input(SW_STOP)); 	// Wait for button to be released
						printf("X\n");				// Send Stop command
					}
				}		
				state = IDLE;	// Return to Idle state			
				break;
		}
		restart_wdt();
	}	
}


BOOLEAN emic2_check(void)	// Check for a ":" from Emic 2 (non-blocking)
{
  restart_wdt();

  if (kbhit())
    if (getch() == ':')
      return 1;

  return 0;		
}


void emic2_init(void) // Make sure the Emic 2 is initialized and ready to go
{
	/*
  	When the Emic 2 powers on, it takes about 3 seconds for it to successfully
    initialize. It then sends a ":" character to indicate it's ready to accept
    commands. If the Emic 2 is already initialized, a CR will also cause it
    to send a ":"
  	*/
	printf("\n");  				// Send a CR in case the system is already up
	while (!emic2_check());		// Wait until Emic is ready
	delay_ms(250);

	printf("V6\n");				// Set volume
	while (!emic2_check());
	delay_ms(250);
}

	
void hardware_init(void) // Configure hardware to a known state
{
	// setup peripherals
	setup_timer_1(T1_DISABLED);
	setup_comparator(NC_NC_NC_NC);
	setup_adc(ADC_OFF);
	setup_adc_ports(NO_ANALOGS);
	setup_vref(FALSE);
	
	setup_wdt(WDT_288MS); 

	port_a_pullups(0b110100); // enable weak pull-ups on GP2/GP4/GP5
	
	delay_ms(250); // Power-up delay
}
